// ===========================================================================
//	CPPStarterApp.cp 		1994-1997 Metrowerks Inc. All rights reserved.
// ===========================================================================
//
//	This file contains the starter code for a basic PowerPlant application

#include "CPPStarterApp.h"

#include <LGrowZone.h>
#include <LWindow.h>
#include <PP_Messages.h>
#include <PP_Resources.h>
#include <PPobClasses.h>
#include <UDrawingState.h>
#include <UMemoryMgr.h>
#include <URegistrar.h>

#include <UControlRegistry.h>
#include <UGraphicUtils.h>
#include <UEnvironment.h>

#ifndef __APPEARANCE__
#include <Appearance.h>
#endif

#include <LProgressBar.h>

// put declarations for resource ids (ResIDTs) here

const ResIDT	window_Sample		= 1;	// EXAMPLE
const ResIDT	about_box			= 2;


class OurWindow: public LWindow, public LListener, public LPeriodical {

public:
	enum { class_ID = 'www1' };

	OurWindow() : LWindow() {}
	OurWindow( LStream *inStream) : LWindow( inStream )
			, mSending(false), mIncrement(1), mPaused(true)
			{}
	
	virtual Boolean		ObeyCommand(
								CommandT			inCommand,
								void				*ioParam);
	virtual void	ListenToMessage(
							MessageT		inMessage,
							void			*ioParam);
							
	virtual	void	SpendTime(
							const EventRecord		&inMacEvent);
							
	virtual	Boolean		HandleKeyPress(			const EventRecord	&inKeyEvent);
private:
	bool		mSending;
	SInt16		mIncrement;
	bool		mPaused;
	
};


// ===========================================================================
//		 Main Program
// ===========================================================================

int main(void)
{
									// Set Debugging options
	SetDebugThrow_(debugAction_Alert);
	SetDebugSignal_(debugAction_Alert);

	InitializeHeap(3);				// Initialize Memory Manager
									// Parameter is number of Master Pointer
									//   blocks to allocate
	
									// Initialize standard Toolbox managers
	UQDGlobals::InitializeToolbox(&qd);
	
									// Initialize PP environment internals
	UEnvironment::InitEnvironment();
	
	new LGrowZone(20000);			// Install a GrowZone function to catch
									//    low memory situations.

	CPPStarterApp	theApp;			// replace this with your App type
	theApp.Run();
	
	return 0;
}


// ---------------------------------------------------------------------------
//		 CPPStarterApp 			// replace this with your App type
// ---------------------------------------------------------------------------
//	Constructor

CPPStarterApp::CPPStarterApp()
{
	if ( UEnvironment::HasFeature( env_HasAppearance ) ) {
		::RegisterAppearanceClient();
	}

	// Register functions to create core PowerPlant classes
	
	RegisterAllPPClasses();
	
	// Register the Appearance Manager/GA classes
	
	UControlRegistry::RegisterClasses();
	
	RegisterClass_(OurWindow);

}


// ---------------------------------------------------------------------------
//		 ~CPPStarterApp			// replace this with your App type
// ---------------------------------------------------------------------------
//	Destructor
//

CPPStarterApp::~CPPStarterApp()
{
}

// ---------------------------------------------------------------------------
//		 StartUp
// ---------------------------------------------------------------------------
//	This function lets you do something when the application starts up
//	without a document. For example, you could issue your own new command.

void
CPPStarterApp::StartUp()
{
	ObeyCommand(cmd_New, nil);		// EXAMPLE, create a new window
}

// ---------------------------------------------------------------------------
//		 ObeyCommand
// ---------------------------------------------------------------------------
//	Respond to commands

Boolean
CPPStarterApp::ObeyCommand(
	CommandT	inCommand,
	void		*ioParam)
{
	Boolean		cmdHandled = true;

	switch (inCommand) {
	
		// Deal with command messages (defined in PP_Messages.h).
		// Any that you don't handle will be passed to LApplication
 			
		case cmd_New:
										// EXAMPLE, create a new window
			OurWindow		*theWindow;
			theWindow = (OurWindow *) LWindow::CreateWindow(window_Sample, this);	
			UReanimator::LinkListenerToControls( theWindow, theWindow, window_Sample);
			theWindow->Show();
			break;



		default:
			cmdHandled = LApplication::ObeyCommand(inCommand, ioParam);
			break;
	}
	
	return cmdHandled;
}

// ---------------------------------------------------------------------------
//		 FindCommandStatus
// ---------------------------------------------------------------------------
//	This function enables menu commands.
//

void
CPPStarterApp::FindCommandStatus(
	CommandT	inCommand,
	Boolean		&outEnabled,
	Boolean		&outUsesMark,
	Char16		&outMark,
	Str255		outName)
{

	switch (inCommand) {
	
		// Return menu item status according to command messages.
		// Any that you don't handle will be passed to LApplication

		case cmd_New:					// EXAMPLE
			outEnabled = true;			// enable the New command
			break;

		default:
			LApplication::FindCommandStatus(inCommand, outEnabled,
												outUsesMark, outMark, outName);
			break;
	}
}

void
CPPStarterApp::ShowAboutBox()
{
	OurWindow		*theWindow;
	theWindow = (OurWindow *) LWindow::CreateWindow(about_box, this);	
	UReanimator::LinkListenerToControls( theWindow, theWindow, about_box);
	theWindow->Show();
}

// ---------------------------------------------------------------------------
//		 ObeyCommand
// ---------------------------------------------------------------------------
//	Respond to commands

Boolean
OurWindow::ObeyCommand(
	CommandT	inCommand,
	void		*ioParam)
{
	Boolean		cmdHandled = true;

	switch (inCommand) {
	
		default:
			cmdHandled = LWindow::ObeyCommand(inCommand, ioParam);
			break;
	}
	
	return cmdHandled;
}

// ---------------------------------------------------------------------------
void	OurWindow::ListenToMessage(
							MessageT		inMessage,
							void			*ioParam) 							
{
#pragma unused(ioParam)

	switch( inMessage )
	{
		case 'but1':
			LButton* but = (LButton*) FindPaneByID('but1');
			if ( !mSending )
				{
				but->SetDescriptor("\pStop");
				StartIdling();
				}
			else
				{
				but->SetDescriptor("\pTransmit");
				StopIdling();

				LProgressBar* pBar = (LProgressBar*) FindPaneByID('PBAR');
				pBar->SetValue( 0 );
				}
				mSending = !mSending;
				
				LPane*  radioGroup = FindPaneByID('RBT ');
				mSending ? radioGroup->Disable() : radioGroup->Enable();
			break;

		case 'but2':
			DoClose();
			break;
					
		case 'RBT2':
			if ( * (long*) ioParam==1 )
				mIncrement = 20;	
			else
				mIncrement  = 1;
			break;
		default:
			break;
	}
}

//---------------------------------------------------
void OurWindow::SpendTime(
							const EventRecord		&inMacEvent)
{
#pragma unused(inMacEvent)

	LProgressBar* pBar = (LProgressBar*) FindPaneByID('PBAR');
	
	if ( !mPaused )
		pBar->SetValue( pBar->GetValue() + mIncrement );
	
	if (pBar->GetValue()==pBar->GetMaxValue())
		{
		pBar->BroadcastMessage('but1',nil);
		}
	
}

//---------------------------------------------------
Boolean
OurWindow::HandleKeyPress(
	const EventRecord	&inKeyEvent)
{
	Boolean	keyHandled = false;
	if (mSuperCommander != nil) {
		keyHandled = mSuperCommander->ProcessKeyPress(inKeyEvent);
	}
	if (!keyHandled)
		{
		keyHandled = true;
		LPane*  text = FindPaneByID('STXT');
		
		switch ( inKeyEvent.message & 0x00FF )
			{
			case 'f': 
				text->SetDescriptor( "\pIncreasing Wattage");
				break;
			case 'a':
				LButton* but = (LButton*) FindPaneByID('but1');
				mPaused = !mPaused;
				
				if (!sIdlerQ || sIdlerQ->FetchIndexOf(this) == LArray::index_Bad) 
					mPaused ? but->Disable() : but->Enable();
					
				text->SetDescriptor( mPaused ? "\pNo Receiver" : "\pConnection Established");
				break;
			default:
				keyHandled = false;
			}
		}
		
	return keyHandled;
}
